import { 
  users, audits, scanResults, services,
  type User, type InsertUser, 
  type Audit, type InsertAudit,
  type ScanResult, type InsertScanResult,
  type Service, type InsertService
} from "@shared/schema";

export interface IStorage {
  // User operations
  getUser(id: number): Promise<User | undefined>;
  getUserByUsername(username: string): Promise<User | undefined>;
  getUserByEmail(email: string): Promise<User | undefined>;
  createUser(user: InsertUser): Promise<User>;
  updateUserCredits(id: number, credits: number): Promise<User | undefined>;

  // Audit operations
  createAudit(userId: number, audit: InsertAudit): Promise<Audit>;
  getAudit(id: number): Promise<Audit | undefined>;
  getUserAudits(userId: number): Promise<Audit[]>;
  updateAuditProgress(id: number, progress: number, status?: string): Promise<Audit | undefined>;
  updateAuditResults(id: number, results: any, securityScore: number, vulnerabilities: number, criticalIssues: number): Promise<Audit | undefined>;

  // Scan operations
  createScanResult(userId: number, scan: InsertScanResult & { threatScore: number; status: string; findings: any }): Promise<ScanResult>;
  getUserScanResults(userId: number): Promise<ScanResult[]>;

  // Service operations
  createService(userId: number, service: InsertService): Promise<Service>;
  getUserServices(userId: number): Promise<Service[]>;
}

export class MemStorage implements IStorage {
  private users: Map<number, User>;
  private audits: Map<number, Audit>;
  private scanResults: Map<number, ScanResult>;
  private services: Map<number, Service>;
  private currentUserId: number;
  private currentAuditId: number;
  private currentScanId: number;
  private currentServiceId: number;

  constructor() {
    this.users = new Map();
    this.audits = new Map();
    this.scanResults = new Map();
    this.services = new Map();
    this.currentUserId = 1;
    this.currentAuditId = 1;
    this.currentScanId = 1;
    this.currentServiceId = 1;

    // Create demo user
    this.createUser({
      username: "admin",
      email: "admin@cyberters.com",
      password: "admin123"
    });
  }

  async getUser(id: number): Promise<User | undefined> {
    return this.users.get(id);
  }

  async getUserByUsername(username: string): Promise<User | undefined> {
    return Array.from(this.users.values()).find(user => user.username === username);
  }

  async getUserByEmail(email: string): Promise<User | undefined> {
    return Array.from(this.users.values()).find(user => user.email === email);
  }

  async createUser(insertUser: InsertUser): Promise<User> {
    const id = this.currentUserId++;
    const user: User = {
      ...insertUser,
      id,
      credits: 100,
      subscriptionPlan: "basic",
      createdAt: new Date(),
    };
    this.users.set(id, user);
    return user;
  }

  async updateUserCredits(id: number, credits: number): Promise<User | undefined> {
    const user = this.users.get(id);
    if (user) {
      user.credits = credits;
      this.users.set(id, user);
      return user;
    }
    return undefined;
  }

  async createAudit(userId: number, insertAudit: InsertAudit): Promise<Audit> {
    const id = this.currentAuditId++;
    const audit: Audit = {
      ...insertAudit,
      id,
      userId,
      status: "pending",
      progress: 0,
      results: null,
      securityScore: null,
      vulnerabilities: null,
      criticalIssues: null,
      createdAt: new Date(),
      completedAt: null,
    };
    this.audits.set(id, audit);
    return audit;
  }

  async getAudit(id: number): Promise<Audit | undefined> {
    return this.audits.get(id);
  }

  async getUserAudits(userId: number): Promise<Audit[]> {
    return Array.from(this.audits.values()).filter(audit => audit.userId === userId);
  }

  async updateAuditProgress(id: number, progress: number, status?: string): Promise<Audit | undefined> {
    const audit = this.audits.get(id);
    if (audit) {
      audit.progress = progress;
      if (status) audit.status = status;
      this.audits.set(id, audit);
      return audit;
    }
    return undefined;
  }

  async updateAuditResults(id: number, results: any, securityScore: number, vulnerabilities: number, criticalIssues: number): Promise<Audit | undefined> {
    const audit = this.audits.get(id);
    if (audit) {
      audit.results = results;
      audit.securityScore = securityScore;
      audit.vulnerabilities = vulnerabilities;
      audit.criticalIssues = criticalIssues;
      audit.status = "completed";
      audit.completedAt = new Date();
      this.audits.set(id, audit);
      return audit;
    }
    return undefined;
  }

  async createScanResult(userId: number, scan: InsertScanResult & { threatScore: number; status: string; findings: any }): Promise<ScanResult> {
    const id = this.currentScanId++;
    const scanResult: ScanResult = {
      ...scan,
      id,
      userId,
      createdAt: new Date(),
    };
    this.scanResults.set(id, scanResult);
    return scanResult;
  }

  async getUserScanResults(userId: number): Promise<ScanResult[]> {
    return Array.from(this.scanResults.values()).filter(scan => scan.userId === userId);
  }

  async createService(userId: number, insertService: InsertService): Promise<Service> {
    const id = this.currentServiceId++;
    const service: Service = {
      ...insertService,
      id,
      userId,
      status: "active",
      createdAt: new Date(),
    };
    this.services.set(id, service);
    return service;
  }

  async getUserServices(userId: number): Promise<Service[]> {
    return Array.from(this.services.values()).filter(service => service.userId === userId);
  }
}

export const storage = new MemStorage();
